/* Some constants for our code */
#define ETH_IP		0x800	; Ethernet type for IP 
#define IPT_TCP		6	; IP type for TCP
#define TCP_WWW		8080	; Destinatin port for WWW
#define FRAG_OFF	0x1FFF	; mask for offset portion of fragment bytes


/* We include a null initialization for microblock style */
#macro WWBumpInit[]
#endm



#macro WWBump[]
xbuf_alloc[$$hdr,6]			; Need 6 SDRAM registers

	.local ifn
	  DL_GetInputPort[ifn]		; Get input port.  (in one of dl_reg*)
	  alu [ ifn, ifn, XOR, 1 ]	; XOR is much cheaper than branches
	  DL_SetOutputPort[ifn]		; Set output port.  (in one of dl_reg*)
	.endlocal


	.local base off
	  Buf_GetData[base, dl_buffer_handle]		; Get the base SDRAM ptr
          DL_GetBufferOffset[off]			; pkt offset is in bytes
	  alu_shf[off, --, B, off, >>3]			; set off in Q-words
	  sdram[read, $$hdr0, base, off, 3], ctx_swap	; read 3 Q-words=6 regs
	.endlocal


	.local etype
	  immed[etype, ETH_IP]
	  alu_shf[ --, etype, -, $$hdr3, >>16]	; only 2nd operand is shifted
	  br!=0[NotWebCtl#]
	.endlocal


	br!=byte[$$hdr5, 0, IPT_TCP, NotWebCtl#]


	.local thoff

	  .local base boff
            ld_field_w_clr[thoff, 0001, $$hdr3, >>6]	; shift before mask
            .local mask
              immed[mask, 0x3c]		; Mask out upper and lower 2 bits
              alu [ thoff, thoff, AND, mask ]
            .endlocal


            alu[thoff, thoff, +, 14]
	    Buf_GetData[base, dl_buffer_handle]
            DL_GetBufferOffset[boff]		
	    alu[boff, boff, +, thoff]		; Add dport off to frame off
	    alu_shf[boff, --, B, boff, >>3]	; convert to Q-Words
	    sdram[read, $$hdr0, base, boff, 3], ctx_swap	; read 24 bytes

	  .endlocal


          alu[ thoff, thoff, AND, 0x7 ]
	  alu[ --, thoff, -, 4]
	  br>=0[SecondWord#]

	.endlocal


	.local sport dport flags

FirstWord#:

	  ld_field_w_clr[sport, 0011, $$hdr0]
	  ld_field_w_clr[dport, 0011, $$hdr1, >>16]	; shift before mask
	  ld_field_w_clr[flags, 0001, $$hdr3]
	  br[GotFields#]

SecondWord#:

	  ld_field_w_clr[sport, 0011, $$hdr1]
	  ld_field_w_clr[dport, 0011, $$hdr2, >>16]	; shift before mask
	  ld_field_w_clr[flags, 0001, $$hdr4]

GotFields#:

	  /* We check for SYN and FIN packets TO port 80 and RST packets */
	  /* FROM port 80 */
	  .local wprt
	    immed[wprt, TCP_WWW]		; Check for port 80 
	    alu[--, dport, -, wprt]		; Check destination port
	    br!=0[CkSrcPort#]

	    alu[--, flags, AND, 0x3 ]		; if its a Syn or Fin: exception
	    br!=0[IsWebCtl#]
	    br[NotWebCtl#]

CkSrcPort#:
	    alu[--, dport, -, wprt]		; Is the source port 80?
	    br!=0[NotWebCtl#]

	    alu[--, flags, AND, 0x4]		; We only care about RST
	    br=0[NotWebCtl#]
	  .endlocal

IsWebCtl#:
	  DL_SetExceptionCode[flags]

	.endlocal


	.local ace_tag
	  immed32[ace_tag, WWBUMP_TAG]
	  DL_SetAceTag[ace_tag]
	.endlocal


	/* Finally set our return code to IX_EXCEPTION.  The dispatch loop */
	/* will actually raise the exception itself */
	immed[dl_next_block, IX_EXCEPTION]
	br[End#]

NotWebCtl#:

	/* If the packet we got is not a web packet then we just note that */
	/* the next microblock should process it.  dl_next_block is our */
	/* return code */
	immed32[dl_next_block, 1]

End#:

xbuf_free[$$hdr]
#endm
