/* table.c -- connection tables for TCP connections */

#include <table.h>
#include <wwbcc.h>	/* for Webcnt */

#define TABSIZ	1024
#define HTSIZ	1543

struct __tab
{
    struct hkey		key;
    int			state;
    int			next;
    int			prev;
} 
Tab[TABSIZ];

int HT[HTSIZ];


#define TS_CLOSED	0
#define TS_OPEN		1


int lookup(struct hkey *hkp);



unsigned int hash(struct hkey *hkp, unsigned int mod)
{ 
    unsigned int h, g;
    unsigned char *cp, *end;

    end = (char *)(hkp + 1);

    h = 0; 
    for ( cp = (char *)hkp ; cp < end ; ++cp ) 
    {
        h = (h << 4) + *cp;
        if ( (g = 0xf0000000 & h) )
        { 
            h ^= g >> 24;
            h ^= g;
        }
    }
    
    return h % mod;
}




void inittab()
{ 
    int i;

    for ( i = 0 ; i < TABSIZ ; i++ )
    {
        Tab[i].state = TS_CLOSED;
        Tab[i].next  = -1;
        Tab[i].prev  = -1;
    }

    for ( i = 0 ; i < HTSIZ ; i++ )
        HT[i] = -1;
}




int newentry()
{ 
    int i;

    for ( i = 0 ; i < TABSIZ ; ++i ) 
    { 
        if ( Tab[i].state == TS_CLOSED )
            return i;
    }

    return -1;
}




int lookup(struct hkey *hkp)
{ 
    int te, i;
    struct hkey k = *hkp;

    te = hash(&k, HTSIZ);
    for ( i = HT[te] ; (i >= 0) && memcmp(&Tab[i].key, &k, sizeof(k)) ; 
          i = Tab[i].next) ; 

    return i;
}




void markopen(struct hkey *hkp)
{
    int te, i;

    if ( (i = lookup(hkp)) < 0 )
    {
        i = newentry();
        if ( i < 0 )
            return;

        te = hash(hkp, HTSIZ);
        Tab[i].next  = HT[te];
        Tab[i].prev  = -(te + 1);
        if ( HT[te] >= 0 )
            Tab[HT[te]].prev = i;
        memcpy(&Tab[i].key, hkp, sizeof(*hkp));
        HT[te] = i;
        Webcnt++;
    }

    Tab[i].state = TS_OPEN;
}




void markclosed(struct hkey *hkp)
{
    int i, te;

    if ( (i = lookup(hkp)) >= 0 )
    {
        if ( Tab[i].next >= 0 )
            Tab[Tab[i].next].prev = Tab[i].prev;
        if ( Tab[i].prev >= 0 )
            Tab[Tab[i].prev].next = Tab[i].next;
        else
        {
            te     = -Tab[i].prev - 1;
            HT[te] = Tab[i].next;
        }

        Tab[i].state = TS_CLOSED;
	Tab[i].next  = -1;
	Tab[i].prev  = -1;
        Webcnt--;
    }
}
