/* cc.c -- wwbump crosscall functions */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <wwbump.h>
#include <wwbcc.h>
#include "wwbump_sk_c.h"
#include "wwbump_cc_c.h"


unsigned int Webcnt;		/* Stores the actual count of web packets */
unsigned char *Sptr;


ix_error cc_init(struct wwbump *wwb)
{
    CC_VMT_wwbump *vmt = 0;
    ix_cap *capp;
    ix_error e;
    RmImportedSymbol sym;
    int i;


    e = RmMalloc(RM_SCRATCH, &Sptr, 1);
    if ( e ) 
        return ix_error_new(0,0,e,"Error in RmMalloc()\n");

    e = RmGetPhysOffset(RM_SCRATCH, Sptr, &Webcnt);
    if ( e ) 
        return ix_error_new(0,0,e,"Error in RmGetPhysOffset()\n");

    e = RmWrite(RM_SCRATCH, Webcnt, 0, 0);
    if ( e ) 
        return ix_error_new(0,0,e,"Error in RmWrite()\n");

    for ( i = 0 ; i < 5 ; ++i ) 
    { 
        if ( (1 << i) & wwb->ue )
        { 
            sym.meNumber = i;
            strncpy(sym.name, PCADDR_STR, MAX_SIZE_OF_IMPORTED_SYMBOL);
	    sym.name[MAX_SIZE_OF_IMPORTED_SYMBOL - 1] = '\0';
            sym.value = Webcnt;

            e = RmUengPatchSymbols(i, &sym, 1);
	    if ( e ) 
                return ix_error_new(0,0,e,"Error in RmUengPatchSymbols()\n");
        }
    }

    /*
      Steps for setting up a crosscall server :
       - Initialize an ix_base_t structure to 0
       - Get the OMS communications access point (CAP) of the ACE
       - Call the crosscall initialization function
    */
    memset(&wwb->ccbase, 0, sizeof(wwb->ccbase));
    ix_ace_to_cap(&wwb->ace, &capp);
    e = sk_wwbump_init(&wwb->ccbase, capp);
    if (e)
        return ix_error_new(0,0,e,"Error in sk_wwbump_init()\n");

    /*
      Steps to retarget incomming crosscalls to our function instead
      of the default : 
       - Get a pointer to the CrossCall Virtual Method Table 
       - retarget the function pointers in that table to our functions
    */
    e = getCCVMT_wwbump(&wwb->ccbase, &vmt);
    if (e) 
        return ix_error_new(0,0,e,"Error in getCCVMT_wwbump()\n");
    vmt->_pCC_wwbump_getcnt = getcnt;

    return 0;
}


ix_error cc_fini(struct wwbump *wwb)
{
    ix_error e;
    /* Crosscall base finalization function */
    e = sk_wwbump_fini(&wwb->ccbase);
    if ( e )
        return ix_error_new(0,0,e,"Error in sk_wwbump_fini()\n");

    e = RmFree(Sptr);
    if ( e ) 
        return ix_error_new(0,0,e,"Error in RmFree()\n");

    return 0;
}


/* This function is called every time someone makes a crosscall */
ix_error getcnt(ix_base_t* bp, long* rv)
{
    unsigned int val, dummy;
    ix_error e;
    (void)bp;	/* prevents compiler warnings */

    e = RmRead(RM_SCRATCH, Webcnt, &val, &dummy);
    if ( e ) 
    {
        ix_error_dump(stderr, e);
        return ix_error_new(0,0,e,"getcnt: Error in RmRead()\n");
    }

    *rv = val;

    /* return 0 for success */
    return 0;
}
